/*
 * Created on October 26, 2005, 4:58 PM
 */

package com.sunlabs.archsim.fleet.ships.selector;
 

import java.util.List;

import com.sunlabs.archsim.runner.Runner;
import com.sunlabs.archsim.runner.OperationLoader;
import com.sunlabs.archsim.base.hierarchy.Entities;
import com.sunlabs.archsim.engine.Director;


import com.sunlabs.archsim.fleet.ships.testing.DataSink;

import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSource;
import com.sunlabs.archsim.gasp.State;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;
import junit.textui.TestRunner;

/*
 * Copyright ??? 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *
 */

/**
 *
 * @author mh193489
 */
public class TestSelector extends TestCase {
    
    private static String fileName= "src/"
            + TestSelector.class.getPackage().getName().replace('.', '/')
            + "/SelectorTestScript.xml";
    
    private static Director director;
    
    
    private static final String SELECT_SOURCE= "SelectSource";
    private static final String IN1_SOURCE= "In1Source";
    private static final String IN2_SOURCE= "In2Source";
    private static final String DATA_SINK= "DataSink";

    /* reference to the DataSink SHIP for result inspection */
    private static DataSink dataSink;
    
    
    protected void setUp() {
        
    }
    
    protected void tearDown() {
        // clean up after tests
        //director.closeJournal();
        director.masterClear();
    }
    
    private void runSimulation() {
        // initialize the tests
        director= Director.getInstance();
        
        OperationLoader ldr= new OperationLoader();
        List ops= ldr.load(fileName);
        
        Runner r= new Runner();
        
        r.perform(ops);
        
        director.journal("Finished Test run");
        
        /* get the reference to current DataSink SHIP instance */
        dataSink= (DataSink)Entities.find(DATA_SINK);
    }
    
    public static Test suite() {
        return new TestSuite(TestSelector.class);
    }
    
    public static void main(String[] args) {
        // run the tests
        TestRunner.run(TestSelector.class);
    } // main
    
    /* Tests for the data interface */
    
   
    public void testValidInputs() {
        // for all valid inputs
        PayloadSource.add(SELECT_SOURCE, new Data(1));
        PayloadSource.add(SELECT_SOURCE, new Data(2));
        PayloadSource.add(IN1_SOURCE, new Data(1));
        PayloadSource.add(IN2_SOURCE, new Data(2));
        
        runSimulation();
        
        // in1 branch
        assertTrue(dataSink.receivedData.get(0).valid);
        assertEquals(dataSink.receivedData.get(0).value, 1);
        // in2 branch
        assertTrue(dataSink.receivedData.get(1).valid);
        assertEquals(dataSink.receivedData.get(1).value, 2);
    }
    
    public void testValidInputsUnselectedBranch() {
        // for not firing not selected branch (together with previous test)
        PayloadSource.add(SELECT_SOURCE, new Data(1));
        PayloadSource.add(SELECT_SOURCE, new Data(2));
        PayloadSource.add(SELECT_SOURCE, new Data(1));
        
        PayloadSource.add(IN1_SOURCE, new Data(1));
        PayloadSource.add(IN1_SOURCE, new Data(2));
        PayloadSource.add(IN1_SOURCE, new Data(5));
        PayloadSource.add(IN2_SOURCE, new Data(3));
        PayloadSource.add(IN2_SOURCE, new Data(4));
        
        runSimulation();
        
        // in1 consumed
        assertTrue(dataSink.receivedData.get(0).valid);
        assertEquals(dataSink.receivedData.get(0).value, 1);
        // in2 consumed, first item which was not consumed when in1 was selected
        assertTrue(dataSink.receivedData.get(1).valid);
        assertEquals(dataSink.receivedData.get(1).value, 3);
        // in1 consumed, first item which was not consumed when in2 was selected
        assertTrue(dataSink.receivedData.get(2).valid);
        assertEquals(dataSink.receivedData.get(2).value, 2);
    }
    
    public void testLASTonSelectAndIn1() {
        // for not firing not selected branch (together with previous test)
        PayloadSource.add(SELECT_SOURCE, new Data(null, false));
        
        PayloadSource.add(IN1_SOURCE, new Data(1));
        PayloadSource.add(IN1_SOURCE, new Data(null, false));
        
        runSimulation();
        
        // in1 consumed
        assertFalse(((State)Entities.find("In1State")).isActive());
        // no output produced
        assertEquals(dataSink.receivedData.size(), 0);
    }
    
    public void testLASTonSelectAndIn2() {
        // for not firing not selected branch (together with previous test)
        PayloadSource.add(SELECT_SOURCE, new Data(null, false));
        
        PayloadSource.add(IN2_SOURCE, new Data(1));
        PayloadSource.add(IN2_SOURCE, new Data(null, false));
        
        runSimulation();
        
        // in1 consumed
        assertFalse(((State)Entities.find("In2State")).isActive());
        // no output produced
        assertEquals(dataSink.receivedData.size(), 0);
    }
    
    public void testConsumingIn1() {
        // for not firing not selected branch (together with previous test)
        PayloadSource.add(SELECT_SOURCE, new Data(null, false));
        
        PayloadSource.add(IN1_SOURCE, new Data(1));
        PayloadSource.add(IN1_SOURCE, new Data(2));
        
        runSimulation();
        
        // in1 consumed
        assertFalse(((State)Entities.find("In1State")).isActive());
        // no output produced
        assertEquals(dataSink.receivedData.size(), 0);
    }
    
    public void testConsumingIn2() {
        // for not firing not selected branch (together with previous test)
        PayloadSource.add(SELECT_SOURCE, new Data(null, false));
        
        PayloadSource.add(IN2_SOURCE, new Data(1));
        PayloadSource.add(IN2_SOURCE, new Data(2));
        
        runSimulation();
        
        // in1 consumed
        assertFalse(((State)Entities.find("In2State")).isActive());
        // no output produced
        assertEquals(dataSink.receivedData.size(), 0);
    }
    
    public void testNoInConsumptionUntilLASTfinished() {
        // for not firing not selected branch (together with previous test)
        PayloadSource.add(SELECT_SOURCE, new Data(null, false));
        
        PayloadSource.add(IN1_SOURCE, new Data(1));
        PayloadSource.add(IN1_SOURCE, new Data(null, false));
        PayloadSource.add(IN1_SOURCE, new Data(2));
        
        PayloadSource.add(IN2_SOURCE, new Data(3));
        PayloadSource.add(IN2_SOURCE, new Data(null, false));
        PayloadSource.add(IN2_SOURCE, new Data(4));
        
        runSimulation();
        
        // select consumed
        assertFalse(((State)Entities.find("SelectState")).isActive());
        // in1 not consumed
        assertTrue(((State)Entities.find("In1State")).isActive());
        assertEquals(((Payload)((State)Entities.find("In1State")).getData()).data.value, 2);
        // in2 not consumed
        assertTrue(((State)Entities.find("In2State")).isActive());
        assertEquals(((Payload)((State)Entities.find("In2State")).getData()).data.value, 4);
        
        // no output produced
        assertEquals(dataSink.receivedData.size(), 0);
    }
    
    public void testNoSelectConsumptionUntilLASTfinished() {
        // for not firing not selected branch (together with previous test)
        PayloadSource.add(SELECT_SOURCE, new Data(null, false));
        PayloadSource.add(SELECT_SOURCE, new Data(2));
        
        PayloadSource.add(IN1_SOURCE, new Data(1));
        PayloadSource.add(IN1_SOURCE, new Data(null, false));
        
        PayloadSource.add(IN2_SOURCE, new Data(3));
        PayloadSource.add(IN2_SOURCE, new Data(4));
        
        runSimulation();
        
        // select consumed
        assertTrue(((State)Entities.find("SelectState")).isActive());
        assertEquals(((Payload)((State)Entities.find("SelectState")).getData()).data.value, 2);
        // in1 not consumed
        assertFalse(((State)Entities.find("In1State")).isActive());
        // in2 not consumed
        assertFalse(((State)Entities.find("In2State")).isActive());
        
        // no output produced
        assertEquals(dataSink.receivedData.size(), 0);
    }
    
}
